//
//  AKBandPassFilterWindow.swift
//  AudioKit
//
//  Autogenerated by scripts by Aurelius Prochazka. Do not edit directly.
//  Copyright (c) 2015 Aurelius Prochazka. All rights reserved.
//

import Foundation
import Cocoa

/// A Window to control AKBandPassFilter in Playgrounds
public class AKBandPassFilterWindow: NSWindow {
    
    private let windowWidth = 400
    private let padding = 30
    private let sliderHeight = 20
    private let numberOfComponents = 2
    
    /// Slider to control centerFrequency
    public let centerFrequencySlider: NSSlider
    /// Slider to control bandwidth
    public let bandwidthSlider: NSSlider
    
    private let centerFrequencyTextField: NSTextField
    private let bandwidthTextField: NSTextField
    
    private var bandPassFilter: AKBandPassFilter
    
    /// Initiate the AKBandPassFilter window
    public init(_ control: AKBandPassFilter) {
        bandPassFilter = control
        let sliderWidth = windowWidth - 2 * padding
        
        centerFrequencySlider = newSlider(sliderWidth)
        bandwidthSlider = newSlider(sliderWidth)
        
        centerFrequencyTextField = newTextField(sliderWidth)
        bandwidthTextField = newTextField(sliderWidth)
        
        let titleHeightApproximation = 50
        let windowHeight = padding * 2 + titleHeightApproximation + numberOfComponents * 3 * sliderHeight
        
        super.init(contentRect: NSRect(x: padding, y: padding, width: windowWidth, height: windowHeight),
            styleMask: NSTitledWindowMask,
            backing: .Buffered,
            `defer`: false)
        self.hasShadow = true
        self.styleMask = NSBorderlessWindowMask | NSResizableWindowMask
        self.movableByWindowBackground = true
        self.level = 7
        self.title = "AKBandPassFilter"
        
        let viewRect = NSRect(x: 0, y: 0, width: windowWidth, height: windowHeight)
        let view = NSView(frame: viewRect)
        
        let topTitle = NSTextField()
        topTitle.stringValue = "AKBandPassFilter"
        topTitle.editable = false
        topTitle.drawsBackground = false
        topTitle.bezeled = false
        topTitle.alignment = NSCenterTextAlignment
        topTitle.font = NSFont(name: "Lucida Grande", size: 24)
        topTitle.sizeToFit()
        topTitle.frame.origin.x = CGFloat(windowWidth / 2) - topTitle.frame.width / 2
        topTitle.frame.origin.y = CGFloat(windowHeight - padding) - topTitle.frame.height
        view.addSubview(topTitle)
        
        makeTextField(centerFrequencyTextField, view: view, below: topTitle, distance: 2,
            stringValue: "Center Frequency: \(bandPassFilter.centerFrequency) Hz")
        makeSlider(centerFrequencySlider, view: view, below: topTitle, distance: 3, target: self,
            action: "updateCenterfrequency",
            currentValue: bandPassFilter.centerFrequency,
            minimumValue: 20,
            maximumValue: 22050)
        
        makeTextField(bandwidthTextField, view: view, below: topTitle, distance: 5,
            stringValue: "Bandwidth: \(bandPassFilter.bandwidth) Cents")
        makeSlider(bandwidthSlider, view: view, below: topTitle, distance: 6, target: self,
            action: "updateBandwidth",
            currentValue: bandPassFilter.bandwidth,
            minimumValue: 100,
            maximumValue: 12000)
        
        self.contentView!.addSubview(view)
        self.makeKeyAndOrderFront(nil)
    }
    
    internal func updateCenterfrequency() {
        bandPassFilter.centerFrequency = centerFrequencySlider.doubleValue
        centerFrequencyTextField.stringValue =
        "Center Frequency \(String(format: "%0.4f", bandPassFilter.centerFrequency)) Hz"
    }
    internal func updateBandwidth() {
        bandPassFilter.bandwidth = bandwidthSlider.doubleValue
        bandwidthTextField.stringValue =
        "Bandwidth \(String(format: "%0.4f", bandPassFilter.bandwidth)) Cents"
    }
    
    /// Required initializer
    required public init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}

